# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

import GridUtils
from Game import GameState

def pieces_disconnected_from_position(grid, pos):
    """ Yields positions of pieces that are *not* connected orthogonally to pos """
    connected = GridUtils.connected_positions(grid, pos, (lambda p,v: v>0), ortho=True)
    return (p for p,v in GridUtils.enumerate_all(grid) if v>0 and p not in connected)

class FormsGameState(GameState):
    def reset(self):
        GameState.reset(self)
        half = self.dimensions()[1] // 2
        for pos in self.itercoords():
            self.grid[pos] = 1 if pos[1]<half else 2

    # this is slow because of the large number of possible moves and calls to connected_positions
    # only 8x8 can be computed in a reasonable time, and AI can only use a depth of 1
    def successors(self):
        pnum, oppnum = self.current_player, self.next_player()
        for ppos in self.enumerate_coords_with_value(pnum):
            for opos in self.enumerate_coords_with_value(oppnum):
                nextgrid = self.grid.copy()
                nextgrid[opos] = pnum
                nextgrid[ppos] = 0
                # remove all pieces not connected to opos
                for rpos in pieces_disconnected_from_position(nextgrid, opos):
                    nextgrid[rpos] = 0
                yield ((ppos, opos), self.next_state_with_grid(nextgrid))
                
    def winning_player(self):
        return self.current_player
        
    def animations_for_move(self, move, newstate):
        import PGPieceAnimation
        # capture animation for moving piece
        animations = PGPieceAnimation.move_with_capture_animations(move[0], move[1])
        # animate any disconnected pieces that are removed
        for pos,val in self.enumerate_all():
            if val>0 and newstate.grid[pos]==0 and pos!=move[0]:
                animations.append(PGPieceAnimation.remove_animation(pos, animations[0].end_time()))
        
        return PGPieceAnimation.AnimationGroup(animations)    
        
from GenericAI import GenericAI
class FormsAI (GenericAI):
    # the object is to get rid of all your pieces, so just negate the default utility function which counts pieces
    def game_utility(self, gstate):
        return -GenericAI.game_utility(self, gstate)
